@@ -191,8 +191,8 @@ static void mode_selection_applet_finalize (GObject *object)
 	if (priv->phonet_proxy)
 		g_object_unref (G_OBJECT (priv->phonet_proxy));
 
-	if (priv->dbus_conn)
-		dbus_g_connection_unref (priv->dbus_conn);
+	/* if (priv->dbus_conn)
+		dbus_g_connection_unref (priv->dbus_conn); */ /* Allegedly, hd_status_plugin_item_get_dbus_g_connection () returns a shared connection. */
 
 	G_OBJECT_CLASS (mode_selection_applet_parent_class)->finalize (object);
 }
@@ -210,13 +210,20 @@ static void mode_selection_applet_class_finalize (ModeSelectionAppletClass *klas
 {
 }
 
+static void mode_selection_applet_on_method_call_finished (DBusGProxy *proxy, DBusGProxyCall *call_id, ModeSelectionApplet *plugin)
+{
+	gchar *phone_state = NULL;
+
+	dbus_g_proxy_end_call (proxy, call_id, NULL, G_TYPE_STRING, &phone_state, G_TYPE_INVALID);
+	mode_selection_applet_update_on_phone_state_changed (NULL, phone_state, plugin);
+}
+
 static void mode_selection_applet_init (ModeSelectionApplet *plugin)
 {
-	gchar *phone_state;
 	GtkWidget *image = NULL;
 	ModeSelectionAppletPrivate *priv = MODE_SELECTION_APPLET_GET_PRIVATE (plugin);
 
-	priv->dbus_conn = dbus_g_bus_get (DBUS_BUS_SYSTEM, NULL);
+	priv->dbus_conn = hd_status_plugin_item_get_dbus_g_connection (HD_STATUS_PLUGIN_ITEM (&plugin->parent), DBUS_BUS_SYSTEM, NULL);
 	g_return_if_fail (priv->dbus_conn);
 
 	priv->phonet_proxy = dbus_g_proxy_new_for_name (priv->dbus_conn, NETWORK_BUS_NAME, NETWORK_PATH, NETWORK_INTERFACE);
@@ -227,15 +234,20 @@ static void mode_selection_applet_init (ModeSelectionApplet *plugin)
 
 	priv->status_button = hildon_button_new_with_text (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH, HILDON_BUTTON_ARRANGEMENT_VERTICAL, g_dgettext_localised_or_english ("osso-connectivity-ui", "conn_fi_phone_network_mode", "Network mode"), NULL);
 	hildon_button_set_style (HILDON_BUTTON (priv->status_button), HILDON_BUTTON_STYLE_PICKER);
+
 	if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), "3g2g-applet-signal"))
 		image = gtk_image_new_from_icon_name ("3g2g-applet-signal", HILDON_ICON_SIZE_FINGER);
 	else
 	{
-		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_scale ("/usr/share/icons/hicolor/48x48/hildon/3g2g-applet-signal.png", 48, 48, TRUE, NULL);
-		image = gtk_image_new_from_pixbuf (pixbuf);
-		g_object_unref (pixbuf);
+		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file ("/usr/share/icons/hicolor/48x48/hildon/3g2g-applet-signal.png", NULL);
+		if (G_LIKELY (pixbuf))
+		{
+			image = gtk_image_new_from_pixbuf (pixbuf);
+			g_object_unref (pixbuf);
+		}
 	}
-	hildon_button_set_image (HILDON_BUTTON (priv->status_button), image);
+	if (G_LIKELY (image))
+		hildon_button_set_image (HILDON_BUTTON (priv->status_button), image);
 
 	hildon_button_set_alignment (HILDON_BUTTON (priv->status_button), 0, 0, 0, 0);
 
@@ -246,7 +258,6 @@ static void mode_selection_applet_init (ModeSelectionApplet *plugin)
 	dbus_g_proxy_add_signal (priv->ssc_proxy, SSC_MODEM_STATE_SIG, G_TYPE_STRING, G_TYPE_INVALID);
 	dbus_g_proxy_connect_signal (priv->ssc_proxy, SSC_MODEM_STATE_SIG, G_CALLBACK (mode_selection_applet_update_on_phone_state_changed), plugin, NULL);
 
-	dbus_g_proxy_call (priv->ssc_proxy, SSC_MODEM_GETSTATE_METHOD, NULL, G_TYPE_INVALID, G_TYPE_STRING, &phone_state, G_TYPE_INVALID);
-	mode_selection_applet_update_on_phone_state_changed (NULL, phone_state, plugin);
+	dbus_g_proxy_begin_call (priv->ssc_proxy, SSC_MODEM_GETSTATE_METHOD, (DBusGProxyCallNotify) mode_selection_applet_on_method_call_finished, plugin, NULL, G_TYPE_INVALID);
 }
 
